@@ -24,8 +24,7 @@
 #include "gnome-vfs-async-job-map.h"
 
 #include "gnome-vfs-job.h"
-#include <glib/ghash.h>
-#include <glib/gmessages.h>
+#include <glib.h>
 
 /* job map bits guarded by this lock */
 static GStaticRecMutex async_job_map_lock = G_STATIC_REC_MUTEX_INIT;
@@ -50,7 +49,14 @@ GnomeVFSJob *
 _gnome_vfs_async_job_map_get_job (const GnomeVFSAsyncHandle *handle)
 {
 	_gnome_vfs_async_job_map_assert_locked ();
-	g_assert (async_job_map != NULL);
+	
+	if (async_job_map == NULL) {
+		/* This can happen if cancellations are run after
+		 * shutdown or before any job is run. This is just
+		 * a special case of there handle not being in the
+		 * hash */
+		return NULL;
+	}
 
 	return g_hash_table_lookup (async_job_map, handle);
 }
