@@ -1,5 +1,5 @@
 
-/* Generated data (by glib-mkenums) */
+
 
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
@@ -15,6 +15,7 @@ gnome_vfs_directory_visit_options_get_type (void)
       { GNOME_VFS_DIRECTORY_VISIT_DEFAULT, "GNOME_VFS_DIRECTORY_VISIT_DEFAULT", "default" },
       { GNOME_VFS_DIRECTORY_VISIT_SAMEFS, "GNOME_VFS_DIRECTORY_VISIT_SAMEFS", "samefs" },
       { GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK, "GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK", "loopcheck" },
+      { GNOME_VFS_DIRECTORY_VISIT_IGNORE_RECURSE_ERROR, "GNOME_VFS_DIRECTORY_VISIT_IGNORE_RECURSE_ERROR", "ignore-recurse-error" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("GnomeVFSDirectoryVisitOptions", values);
@@ -153,6 +154,7 @@ gnome_vfs_set_file_info_mask_get_type (void)
       { GNOME_VFS_SET_FILE_INFO_TIME, "GNOME_VFS_SET_FILE_INFO_TIME", "time" },
       { GNOME_VFS_SET_FILE_INFO_ACL, "GNOME_VFS_SET_FILE_INFO_ACL", "acl" },
       { GNOME_VFS_SET_FILE_INFO_SELINUX_CONTEXT, "GNOME_VFS_SET_FILE_INFO_SELINUX_CONTEXT", "selinux-context" },
+      { GNOME_VFS_SET_FILE_INFO_SYMLINK_NAME, "GNOME_VFS_SET_FILE_INFO_SYMLINK_NAME", "symlink-name" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("GnomeVFSSetFileInfoMask", values);
@@ -352,6 +354,8 @@ gnome_vfs_result_get_type (void)
       { GNOME_VFS_ERROR_NAMESERVER, "GNOME_VFS_ERROR_NAMESERVER", "error-nameserver" },
       { GNOME_VFS_ERROR_LOCKED, "GNOME_VFS_ERROR_LOCKED", "error-locked" },
       { GNOME_VFS_ERROR_DEPRECATED_FUNCTION, "GNOME_VFS_ERROR_DEPRECATED_FUNCTION", "error-deprecated-function" },
+      { GNOME_VFS_ERROR_INVALID_FILENAME, "GNOME_VFS_ERROR_INVALID_FILENAME", "error-invalid-filename" },
+      { GNOME_VFS_ERROR_NOT_A_SYMBOLIC_LINK, "GNOME_VFS_ERROR_NOT_A_SYMBOLIC_LINK", "error-not-a-symbolic-link" },
       { GNOME_VFS_NUM_ERRORS, "GNOME_VFS_NUM_ERRORS", "num-errors" },
       { 0, NULL, NULL }
     };
@@ -607,5 +611,5 @@ gnome_vfs_xfer_phase_get_type (void)
   return etype;
 }
 
-/* Generated data ends here */
+
 
