@@ -44,11 +44,6 @@ void             _gnome_vfs_mime_info_mark_user_mime_dir_dirty   (void);
 GnomeVFSResult _gnome_vfs_get_slow_mime_type_internal (const char  *text_uri,
 						       char       **mime_type);
 
-/* Should be exported, but we're in API freeze */
-GnomeVFSMimeApplication *
-gnome_vfs_mime_get_default_application_for_scheme (const char *mime_type,
-						   const char *scheme);
-
 G_END_DECLS
 
 #endif /* GNOME_VFS_MIME_PRIVATE_H */
