@@ -44,11 +44,35 @@
 #ifdef HAVE_UTMP_H
 #include <utmp.h>
 #endif
+#ifdef HAVE_UTIL_H
+#include <util.h>
+#endif
 #include <glib.h>
 #include "gnome-vfs-pty.h"
 
 int _gnome_vfs_pty_set_size(int master, int columns, int rows);
 
+/* Solaris does not have the login_tty() function so implement locally. */
+#ifndef HAVE_LOGIN_TTY
+static int login_tty(int pts)
+{
+#if defined(HAVE_STROPTS_H)
+  /* push a terminal onto stream head */
+  if (ioctl(pts, I_PUSH, "ptem")   == -1) return -1;
+  if (ioctl(pts, I_PUSH, "ldterm") == -1) return -1;
+#endif
+  setsid();
+#if defined(TIOCSCTTY)
+  ioctl(pts, TIOCSCTTY, 0);
+#endif
+  dup2(pts, 0);
+  dup2(pts, 1);
+  dup2(pts, 2);
+  if (pts > 2) close(pts);
+  return 0;
+}
+#endif
+
 /* Reset the handlers for all known signals to their defaults.  The parent
  * (or one of the libraries it links to) may have changed one to be ignored. */
 static void
@@ -376,7 +400,8 @@ _gnome_vfs_pty_fork_on_pty_name(const char *path, int parent_fd, char **env_add,
 {
 	int fd, i;
 	char c;
-	int ready_a[2], ready_b[2];
+	int ready_a[2] = { 0, 0 };
+	int ready_b[2] = { 0, 0 };
 	pid_t pid, grandchild_pid;
 	int pid_pipe[2];
 	int stdin_pipe[2];
