@@ -30,10 +30,7 @@
 #include "gnome-vfs-private-utils.h"
 #include "gnome-vfs-transform.h"
 #include "gnome-vfs-utils.h"
-#include <glib/ghash.h>
-#include <glib/gmessages.h>
-#include <glib/gstrfuncs.h>
-#include <glib/gstring.h>
+#include <glib.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -897,13 +894,30 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
  * @relative_reference: a string representing a possibly relative uri reference.
  * 
  * Create a new uri from @relative_reference, relative to @base. The resolution
- * algorithm follows RFC 2396. For details, see section 5.2 of
- * http://www.ietf.org/rfc/rfc2396.txt .
+ * algorithm in some aspects follows <ulink url="http://www.ietf.org/rfc/rfc2396.txt">RFC
+ * 2396</ulink>, section 5.2, but is not identical due to some extra assumptions GnomeVFS
+ * makes about URIs.
  *
- * In short, if the @base uri ends in '/', @relative_reference is appended to @base,
- * otherwise it replaces the part of @base after the last '/'.
+ * If @relative_reference begins with a valid scheme identifier followed by ':',
+ * it is assumed to refer to an absolute URI, and a #GnomeVFSURI is created from
+ * it using gnome_vfs_uri_new(). 
  *
- * Return value: The new uri.
+ * Otherwise, depending on its precise syntax, it inherits some aspects of the parent URI,
+ * but the parents' fragment and query components are ignored.
+ *
+ * If @relative_reference begins with "//", it only inherits the @base scheme,
+ * if it begins with '/' (i.e. is an absolute path reference), it inherits everything
+ * ecxept the @base path. Otherwise, it replaces the part of @base after the last '/'.
+ *
+ * <note>
+ *  This function should not be used by application authors unless they expect very
+ *  distinct semantics. Instead, authors should use gnome_vfs_uri_append_file_name(),
+ *  gnome_vfs_uri_append_path(), gnome_vfs_uri_append_string() or
+ *  gnome_vfs_uri_resolve_symbolic_link().
+ * </note>
+ *
+ * Return value: A #GnomeVFSURI referring to @relative_reference, or %NULL
+ * if @relative_reference was malformed.
  */
 GnomeVFSURI *
 gnome_vfs_uri_resolve_relative (const GnomeVFSURI *base,
@@ -913,7 +927,7 @@ gnome_vfs_uri_resolve_relative (const GnomeVFSURI *base,
 	char *text_new;
 	GnomeVFSURI *uri;
 
-	g_assert (relative_reference != NULL);
+	g_return_val_if_fail (relative_reference != NULL, NULL);
 
 	if (base == NULL) {
 		text_base = g_strdup ("");
@@ -936,6 +950,38 @@ gnome_vfs_uri_resolve_relative (const GnomeVFSURI *base,
 	return uri;
 }
 
+/**
+ * gnome_vfs_uri_resolve_symbolic_link:
+ * @base: base uri.
+ * @symbolic_link: a string representing a possibly relative or absolute path reference.
+ * 
+ * Create a new uri from @symbolic_link, relative to @base.
+ *
+ * If @symbolic_link begins with a '/', it replaces the path of @base,
+ * otherwise it is appended after the last '/' character of @base.
+ *
+ * Return value: A new #GnomeVFSURI referring to @symbolic_link.
+ *
+ * Since: 2.16
+ */
+GnomeVFSURI *
+gnome_vfs_uri_resolve_symbolic_link (const GnomeVFSURI *base,
+				     const gchar *symbolic_link)
+{
+	GnomeVFSURI *uri;
+
+	g_return_val_if_fail (base != NULL, NULL);
+	g_return_val_if_fail (symbolic_link != NULL, NULL);
+
+	uri = gnome_vfs_uri_dup (base);
+	g_free (uri->text);
+	uri->text = gnome_vfs_resolve_symlink (gnome_vfs_uri_get_path (base) != NULL ?
+					       gnome_vfs_uri_get_path (base) : "/",
+					       symbolic_link);
+
+	return uri;
+}
+
 /**
  * gnome_vfs_uri_ref:
  * @uri: a #GnomeVFSURI.
@@ -1111,7 +1157,10 @@ gnome_vfs_uri_append_path (const GnomeVFSURI *uri,
 {
 	gchar *escaped_string;
 	GnomeVFSURI *new_uri;
-	
+
+	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (path != NULL, NULL);
+
 	escaped_string = gnome_vfs_escape_path_string (path);
 	new_uri = gnome_vfs_uri_append_string (uri, escaped_string);
 	g_free (escaped_string);
@@ -1136,6 +1185,9 @@ gnome_vfs_uri_append_file_name (const GnomeVFSURI *uri,
 {
 	gchar *escaped_string;
 	GnomeVFSURI *new_uri;
+
+	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (filename != NULL, NULL);
 	
 	escaped_string = gnome_vfs_escape_string (filename);
 	new_uri = gnome_vfs_uri_append_string (uri, escaped_string);
@@ -1172,6 +1224,8 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 	GString *string;
 	gchar *result, *tmp;
 
+	g_return_val_if_fail (uri != NULL, NULL);
+
 	string = g_string_new (uri->method_string);
 	g_string_append_c (string, ':');
 
@@ -1228,9 +1282,9 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 		
 		if (top_level_uri->host_port > 0 
 			&& (hide_options & GNOME_VFS_URI_HIDE_HOST_PORT) == 0) {
-			gchar tmp[128];
-			sprintf (tmp, ":%d", top_level_uri->host_port);
-			g_string_append (string, tmp);
+			gchar temp[128];
+			sprintf (temp, ":%u", top_level_uri->host_port);
+			g_string_append (string, temp);
 		}
 
 	}
@@ -1277,9 +1331,14 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
  * gnome_vfs_uri_is_local:
  * @uri: a #GnomeVFSURI.
  * 
- * Check if @uri is a local (native) file system.
+ * Check if @uri is a local URI. Note that the return value of this
+ * function entirely depends on the #GnomeVFSMethod associated with the
+ * URI. It is up to the method author to distinguish between remote URIs
+ * add URIs referring to entities on the local computer.
+ *
+ * Warning, this can be slow, as it does i/o to detect things like NFS mounts.
  * 
- * Return value: %FALSE if @uri is not a local file system, %TRUE otherwise.
+ * Return value: %TRUE if @uri refers to a local entity, %FALSE otherwise.
  */
 gboolean
 gnome_vfs_uri_is_local (const GnomeVFSURI *uri)
@@ -1308,6 +1367,8 @@ gnome_vfs_uri_has_parent (const GnomeVFSURI *uri)
 {
 	GnomeVFSURI *parent;
 
+	g_return_val_if_fail (uri != NULL, FALSE);
+
 	parent = gnome_vfs_uri_get_parent (uri);
 	if (parent == NULL) {
 		return FALSE;
@@ -1431,6 +1492,8 @@ gnome_vfs_uri_get_host_name (const GnomeVFSURI *uri)
 const gchar *
 gnome_vfs_uri_get_scheme (const GnomeVFSURI *uri)
 {
+	g_return_val_if_fail (uri != NULL, NULL);
+
 	return uri->method_string;
 }
 
@@ -1700,6 +1763,9 @@ gnome_vfs_uri_is_parent (const GnomeVFSURI *possible_parent,
 	GnomeVFSURI *item_parent_uri;
 	GnomeVFSURI *item;
 
+	g_return_val_if_fail (possible_parent != NULL, FALSE);
+	g_return_val_if_fail (possible_child != NULL, FALSE);
+
 	if (!recursive) {
 		item_parent_uri = gnome_vfs_uri_get_parent (possible_child);
 
@@ -1826,6 +1892,8 @@ gnome_vfs_uri_extract_short_name (const GnomeVFSURI *uri)
 	gchar *escaped_short_path_name, *short_path_name;
 	const gchar *host_name;
 
+	g_return_val_if_fail (uri != NULL, NULL);
+
 	escaped_short_path_name = gnome_vfs_uri_extract_short_path_name (uri);
 	short_path_name = gnome_vfs_unescape_string (escaped_short_path_name, "/");
 	g_free (escaped_short_path_name);
@@ -1954,7 +2022,7 @@ gnome_vfs_uri_hash (gconstpointer p)
 		HASH_STRING (hash_value, uri_p->text);
 		HASH_STRING (hash_value, uri_p->method_string);
 
-		if (uri_p->parent != NULL) {
+		if (uri_p->parent == NULL) {
 			const GnomeVFSToplevelURI *toplevel;
 
 			toplevel = (const GnomeVFSToplevelURI *) uri_p;
