@@ -24,9 +24,12 @@
 #ifndef GNOME_VFS_VOLUME_MONITOR_PRIVATE_H
 #define GNOME_VFS_VOLUME_MONITOR_PRIVATE_H
 
-#include <glib/gthread.h>
+#include <glib.h>
 #include "gnome-vfs-volume-monitor.h"
-#include "GNOME_VFS_Daemon.h"
+
+#ifdef USE_DAEMON
+#include <dbus/dbus.h>
+#endif
 
 #define CONNECTED_SERVERS_DIR "/desktop/gnome/connected_servers"
 
@@ -50,6 +53,7 @@ struct _GnomeVFSVolumePrivate {
 	char *activation_uri;
 	char *filesystem_type;
 	char *display_name;
+	char *display_name_key;
 	char *icon;
 	
 	gboolean is_user_visible;
@@ -79,6 +83,7 @@ struct _GnomeVFSDrivePrivate {
 	
 	char *activation_uri;
 	char *display_name;
+	char *display_name_key;
 	char *icon;
 	
 	gboolean is_user_visible;
@@ -87,17 +92,18 @@ struct _GnomeVFSDrivePrivate {
 	/* Only for HAL devices: */
 	char *hal_udi;
 	char *hal_drive_udi; /* only available to daemon; not exported */
+	char *hal_backing_crypto_volume_udi; /* only available to daemon; not exported */
 
 	gboolean must_eject_at_unmount;
 };
 
-void _gnome_vfs_volume_set_drive                (GnomeVFSVolume        *volume,
+void gnome_vfs_volume_set_drive_private         (GnomeVFSVolume        *volume,
 						 GnomeVFSDrive         *drive);
-void _gnome_vfs_drive_add_mounted_volume        (GnomeVFSDrive         *drive,
+void gnome_vfs_drive_add_mounted_volume_private (GnomeVFSDrive         *drive,
 						 GnomeVFSVolume        *volume);
-void _gnome_vfs_drive_remove_volume             (GnomeVFSDrive         *drive,
+void gnome_vfs_drive_remove_volume_private      (GnomeVFSDrive         *drive,
 						 GnomeVFSVolume        *volume);
-void _gnome_vfs_volume_unset_drive              (GnomeVFSVolume        *volume,
+void gnome_vfs_volume_unset_drive_private       (GnomeVFSVolume        *volume,
 						 GnomeVFSDrive         *drive);
 void _gnome_vfs_volume_monitor_mounted          (GnomeVFSVolumeMonitor *volume_monitor,
 						 GnomeVFSVolume        *volume);
@@ -118,14 +124,16 @@ void _gnome_vfs_volume_monitor_shutdown (void);
 
 int _gnome_vfs_device_type_get_sort_group (GnomeVFSDeviceType type);
 
-void            gnome_vfs_volume_to_corba   (GnomeVFSVolume         *volume,
-					     GNOME_VFS_Volume       *corba_volume);
-GnomeVFSVolume *_gnome_vfs_volume_from_corba (const GNOME_VFS_Volume *corba_volume,
-					      GnomeVFSVolumeMonitor  *volume_monitor);
-void            gnome_vfs_drive_to_corba    (GnomeVFSDrive          *drive,
-					     GNOME_VFS_Drive        *corba_drive);
-GnomeVFSDrive * _gnome_vfs_drive_from_corba  (const GNOME_VFS_Drive  *corba_drive,
-					      GnomeVFSVolumeMonitor  *volume_monitor);
+#ifdef USE_DAEMON
+GnomeVFSVolume *_gnome_vfs_volume_from_dbus (DBusMessageIter       *iter,
+					     GnomeVFSVolumeMonitor *volume_monitor);
+gboolean        gnome_vfs_volume_to_dbus    (DBusMessageIter       *iter,
+					     GnomeVFSVolume        *volume);
+gboolean        gnome_vfs_drive_to_dbus     (DBusMessageIter       *iter,
+					     GnomeVFSDrive         *drive);
+GnomeVFSDrive * _gnome_vfs_drive_from_dbus  (DBusMessageIter       *iter,
+					     GnomeVFSVolumeMonitor *volume_monitor);
+#endif
 
 GnomeVFSVolume *_gnome_vfs_volume_monitor_find_mtab_volume_by_activation_uri (GnomeVFSVolumeMonitor *volume_monitor,
 									      const char            *activation_uri);
