@@ -28,18 +28,7 @@
 /* Don't put conflicting kernel types in the global namespace: */
 #define __KERNEL_STRICT_NAMES
 
-#include "local_inotify.h"
-#if 0
-#ifdef HAVE_SYS_INOTIFY_H
-/* We don't actually include the libc header, because there has been
- * problems with libc versions that was built without inotify support.
- * Instead we use the local version.
- */
-#include "local_inotify.h"
-#elif defined (HAVE_LINUX_INOTIFY_H)
-#include <linux/inotify.h>
-#endif
-#endif
+#include <sys/inotify.h>
 #include <string.h>
 #include <glib.h>
 #include <libgnomevfs/gnome-vfs-module-shared.h>
@@ -376,8 +365,8 @@ ip_event_callback (ik_event_t *event)
 	if (event->mask & IP_INOTIFY_MASK)
 		ip_event_dispatch (dir_list, pair_dir_list, event);
 
-	/* We have to manage the missing list when we get a DELETE event. */
-	if (event->mask & IN_DELETE_SELF || event->mask & IN_MOVE_SELF)
+	/* We have to manage the missing list when we get a DELETE/UNMOUNT event. */
+	if (event->mask & IN_DELETE_SELF || event->mask & IN_MOVE_SELF || event->mask & IN_UNMOUNT)
 	{
 		/* Add all subscriptions to missing list */
 		g_list_foreach (dir_list, ip_wd_delete, NULL);
