@@ -36,18 +36,23 @@
 
 #include <config.h>
 
-#include <glib/gstrfuncs.h>
+#include <glib/gi18n-lib.h>
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs-cancellable-ops.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-i18n.h>
 #include <libgnomevfs/gnome-vfs-module.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <libxml/xmlmemory.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#include <sys/time.h>
+#include <sys/types.h>
 #include <unistd.h>
 
+#define TEST_CONF_ENV_VARIABLE "GNOME_VFS_TEST_CONFIG_FILE"
+
 typedef struct {
 	char *operation_name;
         int delay;
@@ -60,7 +65,7 @@ typedef struct {
 
 static gboolean properly_initialized;
 
-static char *test_method_name;
+static guchar *test_method_name;
 static GList *settings_list;
 
 static const char * const
@@ -124,7 +129,7 @@ translate_uri (GnomeVFSURI *uri)
 	no_method = strchr (uri_text, ':');
 	
 	if (test_method_name != NULL) {
-	  translated_uri_text = g_strconcat (test_method_name, 
+	  translated_uri_text = g_strconcat ((char *)test_method_name, 
 					     no_method, NULL);
 	} else {
 	  translated_uri_text = NULL;
@@ -174,7 +179,8 @@ start_operation (const char *name,
 	const OperationSettings *settings;
 
 	settings = get_operation_settings (name);
-	usleep (settings->delay * 1000);
+
+	g_usleep (settings->delay * 1000);
 	
 	if (uri != NULL) {
 		*saved_uri = *uri;
@@ -270,15 +276,15 @@ load_settings (const char *filename)
 	if (doc == NULL
 	    || doc->xmlRootNode == NULL
 	    || doc->xmlRootNode->name == NULL
-	    || g_ascii_strcasecmp (doc->xmlRootNode->name, "testmodule") != 0) {
+	    || g_ascii_strcasecmp ((char *)doc->xmlRootNode->name, "testmodule") != 0) {
 		xmlFreeDoc(doc);
 		return FALSE;
 	}
 
-	test_method_name = xmlGetProp (doc->xmlRootNode, "method");
+	test_method_name = xmlGetProp (doc->xmlRootNode, (guchar *)"method");
 	
 	for (node = doc->xmlRootNode->xmlChildrenNode; node != NULL; node = node->next) {
-		name = xmlGetProp (node, "name");
+		name = (char *)xmlGetProp (node, (guchar *)"name");
 		if (name == NULL) {
 			continue;
 		}
@@ -286,19 +292,19 @@ load_settings (const char *filename)
 		operation = g_new0 (OperationSettings, 1);
 		operation->operation_name = name;
 
-		str = xmlGetProp (node, "delay");
+		str = (char *)xmlGetProp (node, (guchar *)"delay");
 		if (str != NULL) {
 			sscanf (str, "%d", &operation->delay);
 		}
 		xmlFree (str);
 
-		str = xmlGetProp(node, "execute_operation");
+		str = (char *)xmlGetProp(node, (guchar *)"execute_operation");
 		if (str != NULL && g_ascii_strcasecmp (str, "FALSE") == 0) {
 			operation->skip = TRUE;
 		}
 		xmlFree (str);
 
-		str = xmlGetProp (node, "result");
+		str = (char *)xmlGetProp (node, (guchar *)"result");
 		if (str != NULL) {
 			operation->override_result = parse_result_text
 				(str, &operation->overridden_result_value);
@@ -376,7 +382,7 @@ do_seek (GnomeVFSMethod *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod *method,
 	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset *offset_return)
+	 GnomeVFSFileSize *offset_return)
 {
 	PERFORM_OPERATION_NO_URI (tell, gnome_vfs_tell ((GnomeVFSHandle *) method_handle, offset_return));
 }
@@ -565,13 +571,23 @@ static GnomeVFSMethod method = {
 GnomeVFSMethod *
 vfs_module_init (const char *method_name, const char *args)
 {
+	char *conf_file;
+
 	LIBXML_TEST_VERSION
 	
-	if (load_settings (PREFIX "/etc/vfs/Test-conf.xml") == FALSE) {
+	conf_file = getenv (TEST_CONF_ENV_VARIABLE);
 
-	  // FIXME: we probably shouldn't use printf to output the message
+	if (conf_file == NULL) {
+		conf_file = GNOME_VFS_PREFIX "/etc/vfs/Test-conf.xml";
+	}
+
+	if (load_settings (conf_file) == FALSE) {
+
+	  /* FIXME: we probably shouldn't use printf to output the message */
 	  printf (_("Didn't find a valid settings file at %s\n"), 
-		  PREFIX "/etc/vfs/Test-conf.xml");
+		  conf_file);
+	  printf (_("Use the %s environment variable to specify a different location.\n"),
+		  TEST_CONF_ENV_VARIABLE);
 	  properly_initialized = FALSE;
 	} else {
 	  properly_initialized = TRUE;
