@@ -27,13 +27,13 @@
 
 #include <config.h>
 
-#include <glib/gmessages.h>
-#include <glib/gstrfuncs.h>
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-private-utils.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 
 #define TEST_ASSERT(expression, message) \
 	G_STMT_START { if (!(expression)) test_failed message; } G_STMT_END
@@ -92,7 +92,7 @@ test_make_canonical_path (const char *input,
 	g_free (output);
 }
 
-const char* test_uris[][2] = 
+static const char* test_uris[][2] = 
 {
 	{ "http://www.gnome.org/", "index.html" },
 	{ "http://www.gnome.org/", "/index.html"},
@@ -622,15 +622,13 @@ main (int argc, char **argv)
 	test_make_canonical ("http:///&", "http:///&");
 	test_make_canonical ("http:///x", "http:///x");
 	test_make_canonical ("pipe:foo", "pipe:foo");
-	test_make_canonical ("eazel-services:///%3F", "eazel-services:///%3F");
-	test_make_canonical ("eazel-services:///%78", "eazel-services:///x");
-	test_make_canonical ("eazel-services:///?", "eazel-services:///?");
-	test_make_canonical ("eazel-services:///&", "eazel-services:///&");
-	test_make_canonical ("eazel-services:///x", "eazel-services:///x");
-
-	test_make_canonical ("eazel-install://anonymous@/product_name=gnucash", "eazel-install://anonymous@/product_name%3Dgnucash");
-	test_make_canonical ("eazel-install://:password@/product_name=gnucash", "eazel-install://:password@/product_name%3Dgnucash");
+	test_make_canonical ("dav:///%3F", "dav:///%3F");
+	test_make_canonical ("dav:///%78", "dav:///x");
+	test_make_canonical ("dav:///?", "dav:///?");
+	test_make_canonical ("dav:///&", "dav:///&");
+	test_make_canonical ("dav:///x", "dav:///x");
 
+	
 	test_make_canonical ("http://www.eazel.com/query?email=email@eazel.com", "http://www.eazel.com/query?email=email@eazel.com");
 
 	/* test proper case-sensitivity handling */
